#include "configuration.h"

#include <qtimer.h>

#include <kapplication.h>      
#include <qstringlist.h>
#include <kconfig.h>a
#include <kdebug.h>


Configuration::Configuration() {
    /// initialises the Configuration object and reads the 
    /// configuration file 
    readSettings();
};

Configuration::~Configuration() {
    m_password.fill(0);
};

/// reads the global parameters from the configuration file
void Configuration::readSettings() {
    const QString confFile="kwlanrc";
    KConfig *conf= new KConfig(confFile,false,false,"config");
    if (!conf) return;
    //read settings
    m_startWpa = conf->readBoolEntry("startwpa",TRUE);
    m_startWpaNoQuestion = conf->readBoolEntry("startwpanoquestion",TRUE);
    m_startMinimized = conf->readBoolEntry("startminimized",FALSE);
    m_pwexpire = conf->readNumEntry("passwordexpiretimeout", 15);
    m_lastSSID = conf->readEntry("lastssid","");
    delete conf;
}



/// saves the global parameters to the configuration file
void Configuration::writeSettings() {
    const QString confFile="kwlanrc";
    KConfig *conf= new KConfig(confFile,false,false,"config");
    if (!conf) return;
    //conf->writeEntry("startdhcp",m_startDhcp);
    conf->writeEntry("startwpa",m_startWpa);
    conf->writeEntry("startwpanoquestion",m_startWpaNoQuestion);
    conf->writeEntry("startminimized",m_startMinimized);
    conf->writeEntry("passwordexpiretimeout", m_pwexpire);
    conf->writeEntry("lastssid",m_lastSSID);
    delete conf;
}

int Configuration::readIpSettings(QString ssid, bool &useDhcp, QString & ipaddress, QString & netmask, QString & gateway, QString & dns1, QString & dns2, QString & domain, bool & dontOverrideGw, bool & dontOverrideDns)
{
    const QString confFile="kwlanrc";
    useDhcp = true;
    KConfig *conf= new KConfig(confFile,false,false,"config");
    if (!conf) return -1;
    // switch 
    if (!conf->hasGroup(ssid)) return -1;
    conf->setGroup(ssid);
    //read settings
    useDhcp = conf->readBoolEntry("usedhcp",TRUE);
    ipaddress = conf->readEntry("ipaddress","");
    gateway = conf->readEntry("gateway","");
    netmask= conf->readEntry("netmask","");
    dns1 = conf->readEntry("dns1","");
    dns2 = conf->readEntry("dns2","");
    domain = conf->readEntry("domain","");
    dontOverrideGw = conf->readBoolEntry("dontoverridegw",FALSE);
    dontOverrideDns = conf->readBoolEntry("dontoverridedns",FALSE);
    delete conf;
    return 0;
}

int Configuration::writeIpSettings(QString ssid, bool useDhcp, QString ipaddress, QString netmask, QString gateway, QString dns1, QString dns2,QString domain, bool dontOverrideGw, bool dontOverrideDns)
{
    const QString confFile="kwlanrc";
    KConfig *conf= new KConfig(confFile,false,false,"config");
    if (!conf) return -1;
    conf->setGroup(ssid);
    conf->writeEntry("usedhcp",useDhcp);
    conf->writeEntry("ipaddress",ipaddress);
    conf->writeEntry("gateway",gateway);
    conf->writeEntry("netmask",netmask);
    conf->writeEntry("dns1",dns1);
    conf->writeEntry("dns2",dns2);
    conf->writeEntry("domain",domain);
    conf->writeEntry("dontoverridegw",dontOverrideGw);
    conf->writeEntry("dontoverridedns",dontOverrideDns);
    delete conf;
    return 0;
}

void Configuration::readDriver(QString interface, QString &driver)
{
    const QString confFile="kwlanrc";
    KConfig *conf= new KConfig(confFile,false,false,"config");
    if (!conf) return;
    conf->setGroup(interface);
    driver = conf->readEntry("driver");
    delete conf;
}

void Configuration::writeDriver(QString interface, QString driver)
{
    const QString confFile="kwlanrc";
    KConfig *conf= new KConfig(confFile,false,false,"config");
    if (!conf) return;
    conf->setGroup(interface);
    conf->writeEntry("driver",driver);
    delete conf;
}


void Configuration::setPassword(const QCString &password)
{
    m_password = password;
    if (m_pwexpire !=0) QTimer::singleShot(m_pwexpire * 60000, this, SLOT( wipePassword() ));
}

const QCString& Configuration::password() const
{
    return m_password;
}

void Configuration::wipePassword()
{
    m_password.fill(0);
}

Configuration& Config() {
    static Configuration conf;
    return conf;
};


