#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include <qstring.h>
/**
	@author Thomas Luebke <tluebke@asterisk2>
*/

#include <qcstring.h>
#include <kconfig.h>


class Configuration: public QObject {
    Q_OBJECT
    
public:
    virtual ~Configuration();
    /// Reads all the configuration data from the application config file.
    void readSettings();
    /// Writes the configuration data to the application config file.
    void writeSettings();
    /// Reads IP Settings for a given SSID
    int readIpSettings(QString ssid, bool &useDhcp, QString  &ipaddress, QString &netmask, QString & gateway, QString & dns1, QString & dns2, QString & domain, bool & dontOverrideGw, bool & dontOverrideDns);
    /// Writes IP Settings for a given SSID
    int writeIpSettings(QString ssid, bool useDhcp, QString ipaddress, QString netmask, QString gateway, QString dns1, QString dns2, QString domain,bool dontOverrideGw,bool dontOverrideDns);
    void readDriver(QString interface, QString &driver);
    void writeDriver(QString interface, QString driver);
    /// Cache the password for m_pwexpire seconds
    void setPassword(const QCString &password);
    /// Retrive cached password
    const QCString& password() const;
    QString m_driver;
    QString m_lastSSID;
    //bool m_startDhcp;
    bool m_startWpa;
    bool m_startWpaNoQuestion;
    bool m_startMinimized;
    int m_pwexpire;
    
    
private:
    Configuration();
    Configuration(const Configuration&);

    // allow this function to create one instance
    friend Configuration& Config();

    //Runtime-only field. Should be empty to indicate no password cached
    QCString m_password;
    
private slots:    
    void wipePassword();
};

// use this function to access the settings
Configuration& Config();
#endif
