

#include "kscandlg.h"

#include "kprofileconfigdlg.h"

#include <qlistview.h>
#include <kmessagebox.h>

KScanDlg::KScanDlg(QWidget* parent, const char* name, bool modal, WFlags fl)
: KScan(parent,name, modal,fl)
{
    m_mainWin = NULL;
}

KScanDlg::~KScanDlg()
{
}

/*$SPECIALIZATION$*/
void KScanDlg::addSelected()
{
    QListViewItem *sel = scanResultsView->selectedItem();
    if (!sel) {
        KMessageBox::sorry(this,"kwlan","No Network selected!");
        return;
    }
    bssSelected( sel);
}

void KScanDlg::bssSelected( QListViewItem * sel )
{
    KProfileConfigDlg *nc = new KProfileConfigDlg();
    if (nc == NULL)
        return;
    nc->setMainWin(m_mainWin);
    nc->paramsFromScanResults(sel);
    nc->show();
    nc->exec();
}

void KScanDlg::getResults()
{
    updateResults();
}

void KScanDlg::scanRequest()
{
    char reply[10];
    size_t reply_len = sizeof(reply);
    
    if (m_mainWin == NULL)
        return;
    
    m_mainWin->ctrlRequest("SCAN", reply, &reply_len);
}

void KScanDlg::updateResults()
{
    char reply[8192];
    size_t reply_len;
    
    if (m_mainWin == NULL)
        return;

    reply_len = sizeof(reply) - 1;
    if (m_mainWin->ctrlRequest("SCAN_RESULTS", reply, &reply_len) < 0)
        return;
    reply[reply_len] = '\0';

    scanResultsView->clear();
    
    QString res(reply);
    QStringList lines = QStringList::split(QChar('\n'), res);
    bool first = true;
    for (QStringList::Iterator it = lines.begin(); it != lines.end(); it++) {
        if (first) {
            first = false;
            continue;
        }
	
        QStringList cols = QStringList::split(QChar('\t'), *it, true);
        QString ssid, bssid, freq, signal, flags;
        bssid = cols.count() > 0 ? cols[0] : "";
        freq = cols.count() > 1 ? cols[1]: "";
        signal = cols.count() > 2 ? cols[2] : "";
        flags = cols.count() > 3 ? cols[3] : "";
        ssid = cols.count() > 4 ? cols[4] : "";
        //work around signal bug (it's about twice as what iwscan gives)
        signal = QString("%1").arg(signal.toInt()/2);
        new QListViewItem(scanResultsView, ssid, bssid, freq, signal, flags);
    }
}

void KScanDlg::setMainWin( KWlanMainWin *mainWin )
{
    m_mainWin  = mainWin;
    updateResults();
    m_timer = new QTimer(this);
    if (!m_timer) return;
    connect(m_timer, SIGNAL(timeout()), SLOT(getResults()));
    m_timer->start(10000, FALSE);
}



#include "kscandlg.moc"


