

#include "kwlanmainwin.h"
#include "wlanlib.h"
#include "kprofileconfigdlg.h"
#include "kstartsupplicantdlg.h"
#include "wpa_ctrl.h"
#include "keventhistorydlg.h"
#include "kscandlg.h"
#include "kuserdatarequestdlg.h"
#include "kwlansettingsdlg.h"
#include "kwlansuprocess.h"


#include <iostream>
#include <kstdaction.h>
#include <kaction.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qapplication.h>
#include <qtimer.h>
#include <qlabel.h>
#include <qaction.h>
#include <unistd.h>
#include <dirent.h>
#include <klocale.h>
#include <qstringlist.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <kaboutapplication.h>
#include <qcstring.h>
#include <kpassdlg.h>
#include <kmessagebox.h>
#include <ktempfile.h>
#include <kstandarddirs.h>
#include <kpassdlg.h>
#include <kdebug.h>

static int str_match(const char *a, const char *b)
{
    return strncmp(a, b, strlen(b)) == 0;
}


// Non Member function
static void kwlan_msg_cb(char *msg, size_t)
{
    /* This should not happen anymore since two control connections are used. */
    printf("missed message: %s\n", msg);
}

KWlanMainWin::KWlanMainWin(QWidget* parent, const char* name, WFlags fl)
    : Kwlan(parent,name,fl)
{
}

int KWlanMainWin::init(QString interface) 
{
    char buf[10];
    size_t len;
    QStringList binDirs;

    // initialise variables
    m_controlInterface = strdup(interface.ascii());
    m_monitorConnection = NULL;
    m_messageNotifier = NULL;
    m_controlConnection = NULL;
    m_scanDlg = NULL;
    m_controlInterfaceDir = strdup ("/var/run/wpa_supplicant");
    m_IpAddress = FALSE;
    m_associated = FALSE;
    m_wpaStarted = FALSE;
    m_eventHistory = NULL;
    m_userDataRequest = NULL;
    m_useDhclient = FALSE;
    m_useDhcpcd = FALSE;
    m_dhcpEnabled = TRUE;
    if (Config().m_startWpa)  m_wpaWarned = TRUE; // Don't warn if we start it anyway
    else m_wpaWarned = FALSE;
    m_ssid = "";
    //get path to wpa_supplicant
    m_wpaPath = getPath("wpa_supplicant");
    if (m_wpaPath.isEmpty()){
        KMessageBox::error(this,i18n("Could not find wpa_supplicant binary. Without it KWlan can't do anything usefull.\n"
                "Please check your wpa_supplicant installation. KWlan will now quit."));
	return -1;
    }
    
    //Check for dhcp client to use
    m_dhcpcdPath = getPath("dhcpcd");
    if (!m_dhcpcdPath.isEmpty()){
        m_useDhcpcd = TRUE;
    } else {
        m_dhclientPath = getPath("dhclient");
        if (!m_dhclientPath.isEmpty())
            m_useDhclient = TRUE;
    }
    if (! m_useDhclient && ! m_useDhcpcd)
        KMessageBox::sorry(this,i18n("Neither dhclient nor dhcpcd found. Dynamic IP configuration will not work."));
    // start systray window
    m_tray = new KWlanTrayWin( this, "KWlan" );
    if (!m_tray) {
        KMessageBox::sorry(this,i18n("Could not create Kicker applet!"));
        fileExit();
    }
    m_tray->show();
    connect( m_tray, SIGNAL( quitSelected() ), this, SLOT( fileExit() ) );
    connect(m_tray, SIGNAL (aboutSelected()),this,SLOT(helpAbout()));
    connect(m_tray, SIGNAL(profileChanged(QString )),this,SLOT(profileSelect(QString )));
    connect(this, SIGNAL(interfaceChanged( const QString& )), m_tray, SLOT(setInterface (const QString &)));
    connect(this, SIGNAL(ipChanged( const QString& )),m_tray, SLOT(setIpAddress(const QString &)));
    connect(this, SIGNAL(statusChanged( bool )),m_tray, SLOT(setConnectionStatus(bool)));
    // Set Status update timer
    m_timer = new QTimer(this);
    if (m_timer){
        connect (m_timer, SIGNAL(timeout()),SLOT(checkConnection()));
        m_timer->start(5000,FALSE);
    }
    //Start WPA Supplicant if configured
    len = sizeof(buf)-1;
    checkConnection();
    if (!m_wpaStarted && Config().m_startWpa){
        KStartSupplicantDlg *startSupplicant = new KStartSupplicantDlg();
	QString driver;
	Config().readDriver(QString(m_controlInterface),driver);
        if (startSupplicant ){
            startSupplicant->setData(&interface,&driver);
            if (Config().m_startWpaNoQuestion&& !driver.isEmpty()){
                startSupplicant->startSupplicant();
            }
            else {
                startSupplicant->show();
                startSupplicant->exec();
            }
            QString wpa_supplicant;
            if (startSupplicant->m_start){
                KWlanSuProcess *startWpa= new KWlanSuProcess(this);
                *startWpa << m_wpaPath << QString("-i%1").arg(startSupplicant->getInterface()) << QString ("-D%1").arg(startSupplicant->getDriver()) << "-c/etc/wpa_supplicant.conf -B";
                startWpa->setDescription(i18n("Start WPA Supplicant"));
                startWpa->start();
            }
            delete startSupplicant;
            m_networkChange = true;
            updateNetworks();
        }
    }
    if (txtStatus->text()=="COMPLETED"){
        m_associated = true;
        emit statusChanged( true);

    } else if (!Config().m_lastSSID.isEmpty()){
        //Wait two seconds to allow wpa_supplicant to start
        
        sleep(2);
        if (openControlConnection(m_controlInterface)!=-1)
            profileSelect(Config().m_lastSSID);
    }
    txtInterface->setText(interface);
    return 0;
}

KWlanMainWin::~KWlanMainWin()
{
    Config().writeSettings();
}

void KWlanMainWin::cleanTemporaryFiles()
{
    resolvconf->unlink();
    if (resolvconf) {
        delete resolvconf;
        resolvconf = 0;
    }
}

/*$SPECIALIZATION$*/
void KWlanMainWin::cbInterfaceChanged(const QString & interface)
{
    kdDebug() << "Interface: " << interface << endl;
    if (!interface.isEmpty())
        openControlConnection( interface.ascii());
}





void KWlanMainWin::editSettings()
{
    KWlanSettingsDlg *settings = new KWlanSettingsDlg();
    if (!settings) return;
    settings->show();
    settings->exec();
}

void KWlanMainWin::releaseIpAddress()
{
    if ((!m_controlInterface) && (!m_dhcpEnabled)) return;

    KWlanSuProcess *Dhclient = new KWlanSuProcess(this);    
	
    if (m_useDhclient)
    {
        *Dhclient << getPath("dhclient") << "-r" << m_controlInterface;
    } else if (m_useDhcpcd){
        *Dhclient << getPath("dhcpcd") << "-k" << m_controlInterface;
    }
	
    Dhclient->setDescription(i18n("Release IP address"));
    Dhclient->start();
    emit ipChanged( QString(""));
}

void KWlanMainWin::receiveMsgs()
{
    char buf[256];
    size_t len;
    
    while (wpa_ctrl_pending(m_monitorConnection)) {
        len = sizeof(buf) - 1;
        if (wpa_ctrl_recv(m_monitorConnection, buf, &len) == 0) {
            buf[len] = '\0';
            processMessage(buf);
        }
    }
}

void KWlanMainWin::processCtrlReq( const char * request )
{
    // processe control request, for example ask for password for private key file.
   if (m_userDataRequest) {
        m_userDataRequest->close();
        delete m_userDataRequest;
    }
    m_userDataRequest = new KUserDataRequestDlg();
    if (m_userDataRequest == NULL)
        return;
    if (m_userDataRequest->setParams(this, request) < 0) {
        delete  m_userDataRequest;
        m_userDataRequest = NULL;
        return;
    }
    m_userDataRequest->show();
    m_userDataRequest->exec();
}

void KWlanMainWin::processMessage(char *message)
{
    char *pos = message, *pos2;
    int priority = 2;
    kdDebug() << message << endl;
    if (*pos == '<') {
        /* skip priority */
        pos++;
        priority = atoi(pos);
        pos = strchr(pos, '>');
        if (pos)
            pos++;
        else
            pos = message;
    }

    WpaMsg wm(pos, priority);
    if (m_eventHistory)
        m_eventHistory->addEvent(wm);
    m_messages.append(wm);
    while (m_messages.count() > 100)
        m_messages.pop_front();
    
    /* Update last message with truncated version of the event */
    if (strncmp(pos, "CTRL-", 5) == 0) {
        pos2 = strchr(pos, str_match(pos, WPA_CTRL_REQ) ? ':' : ' ');
        if (pos2)
            pos2++;
        else
            pos2 = pos;
    } else
        pos2 = pos;
    QString lastmsg = pos2;
    lastmsg.truncate(40);
    //textLastMessage->setText(lastmsg);
    //pingsToStatusUpdate = 0;
    m_networkChange = true;
    
    if (str_match(pos, WPA_CTRL_REQ))
        processCtrlReq(pos + strlen(WPA_CTRL_REQ));
    if (str_match(pos,WPA_EVENT_CONNECTED)){
        // getIpAddress
        if (!m_associated) {
            getIpAddress();
        }
        m_associated = true;
        emit statusChanged( true);
    }
    if (str_match(pos,WPA_EVENT_DISCONNECTED)){
        //if (m_dhcpEnabled && m_associated) releaseIpAddress();
        //m_ssid = "";
        m_associated = false;
        emit statusChanged( false);
    }
}

void KWlanMainWin::toolsLogViewer()
{
    if (m_eventHistory) {
        m_eventHistory->close();
        delete m_eventHistory;
    }

    m_eventHistory = new KEventHistoryDlg();
    if (m_eventHistory == NULL)
        return;
    m_eventHistory->addEvents(m_messages);
    m_eventHistory->show();
    m_eventHistory->exec();
    
}

void KWlanMainWin::scan()
{
    if (m_scanDlg) {
        m_scanDlg->close();
        delete m_scanDlg;
    }

    m_scanDlg = new KScanDlg();
    if (m_scanDlg == NULL)
        return;
    m_scanDlg->setMainWin(this);
    m_scanDlg->show();
    m_scanDlg->exec();
}


void KWlanMainWin::fileStopWpaSupplicant()
{
    char reply[100];
    size_t reply_len;
    reply_len = sizeof(reply);
    ctrlRequest("TERMINATE", reply, &reply_len);
    if (strncmp(reply,"OK",2) != 0) {
        KMessageBox::sorry(this,  i18n("Could not terminate WPA_Supplicant!"));
        return;
    }
    m_wpaStarted = false;
    // User stopped wpa_suppliant, so suppress warning
    m_wpaWarned = true;
    //fileStartWpaAction->setEnabled(true);
    //fileStopWpaAction->setDisabled(true);
}

void KWlanMainWin::getIpAddress()
{
    if (!m_dhcpEnabled){
        if (m_staticIpAddress.isEmpty()) return;
	
        KWlanSuProcess *configurator = new KWlanSuProcess(this);
        configurator->setDescription(i18n("Configure the network"));	
        *configurator << getPath("ifconfig") << m_controlInterface << m_staticIpAddress.ascii();	
	
        if (!m_staticNetmask.isEmpty())
            *configurator << "netmask" << m_staticNetmask.ascii();

	// We can to set up our new route if there is no route (hasGateway()==false) or we have but m_ovrgw is true ()
        bool m_hasGateway = hasGateway();
        if (!m_staticGateway.isEmpty() && (!m_dontOverrideGw || !m_hasGateway)) {
            *configurator << ";";
	    
	    // Delete default route if we already have one
            if (m_hasGateway)
                *configurator << getPath("route") << "del" << "default" << ";";
		
            *configurator << getPath("route") << "add" << "default" << "gw" << m_staticGateway.ascii();
        }    

        if (!m_dontOverrideDns || !hasNameserver()) {
	    // Create temporary file containing DNS configuration	    
            resolvconf = new KTempFile(locateLocal("tmp", "resolv"), ".conf", 0644);
            if (!m_staticDns1.isEmpty())
                *(resolvconf->textStream()) << QString("nameserver %1").arg(m_staticDns1) << endl;
            if (!m_staticDns2.isEmpty())
                *(resolvconf->textStream()) << QString("nameserver %1").arg(m_staticDns2) << endl;
            if (!m_domain.isEmpty())
                *(resolvconf->textStream()) << QString("search %1").arg(m_domain) << endl;
            resolvconf->close();
            connect(configurator, SIGNAL( destroyed() ), this, SLOT( cleanTemporaryFiles() ));

	    // And copy it to /etc fixing permissions (moving wont work if /etc/resolv.conf is a symlink)
            *configurator << ";";
            *configurator << getPath("cp") << resolvconf->name() << "/etc/resolv.conf";	    
            *configurator << ";";
            *configurator << getPath("chown") << "root:root" << "/etc/resolv.conf";
        }

        configurator->start();
    }
    else {
        KWlanSuProcess *Dhclient = new KWlanSuProcess(this);    
	
        if (m_useDhclient)
        {
            //*Dhclient << getPath("dhclient") <<  "-1" << m_controlInterface;
            *Dhclient << getPath("dhclient") << m_controlInterface;
        } else if (m_useDhcpcd){
            *Dhclient << getPath("dhcpcd") << "-nd" << m_controlInterface;
        }
	
        Dhclient->setDescription(i18n("Obtain IP address dynamically"));
        Dhclient->setRestart(true);
        Dhclient->start();
    }
}


void KWlanMainWin::fileStartWpaSupplicant()
{
    QString interface = QString(m_controlInterface);
    KStartSupplicantDlg *startSupplicant = new KStartSupplicantDlg();
    if (!startSupplicant) return;
    QString driver;
    Config().readDriver(QString(m_controlInterface), driver);
    startSupplicant->setData( & interface,& driver);
    startSupplicant->show();
    startSupplicant->exec();
    if (startSupplicant->m_start){
        KWlanSuProcess *startWpa = new KWlanSuProcess(this);
        *startWpa << m_wpaPath << QString("-i%1").arg(QString(m_controlInterface)) << QString ("-D%1").arg(startSupplicant->getDriver()) << "-c/etc/wpa_supplicant.conf -B";
        startWpa->setDescription(i18n("Start WPA Supplicant"));
        startWpa->start();
	//remember last interface driver
	Config().writeDriver(QString(m_controlInterface),startSupplicant->getDriver());
        //fileStartWpaAction->setDisabled(true);
        //fileStopWpaAction->setEnabled(true);
    }
    if (!Config().m_lastSSID.isEmpty()){
        //Wait two seconds to allow wpa_supplicant to start
        sleep(2);
        if (openControlConnection(startSupplicant->getInterface())!=-1)
           profileSelect(Config().m_lastSSID);
    }
    //updateInterfaces();
}

void KWlanMainWin::triggerUpdate()
{
    updateStatus();
    m_networkChange = true;
    updateNetworks();
}

int KWlanMainWin::openControlConnection(const char *interfaceName)
{
    char *cfile;
    int flen;

    if (interfaceName) {
        if (interfaceName != m_controlInterface) {
            delete m_controlInterface;
            m_controlInterface = strdup(interfaceName);
        }
        emit interfaceChanged( m_controlInterface);
    } else {
	struct dirent *dent;
	DIR *dir = opendir(m_controlInterfaceDir);
        delete m_controlInterface;
        m_controlInterface = NULL;
        if (dir) {
            while ((dent = readdir(dir))) {
                if (strcmp(dent->d_name, ".") == 0 ||
                    strcmp(dent->d_name, "..") == 0)
                    continue;
                //printf("Selected interface '%s'\n", dent->d_name);
                m_controlInterface = strdup(dent->d_name);
                emit interfaceChanged( m_controlInterface);
                break;
            }
            closedir(dir);
        }
    }
    
    if (m_controlInterface == NULL)
        return -1;

    flen = strlen(m_controlInterfaceDir) + strlen(m_controlInterface) + 2;
    cfile = (char *) malloc(flen);
    if (cfile == NULL)
        return -1;
    snprintf(cfile, flen, "%s/%s", m_controlInterfaceDir, m_controlInterface);

    if (m_controlConnection) {
        wpa_ctrl_close(m_controlConnection);
        m_controlConnection = NULL;
    }

    if (m_monitorConnection) {
        delete m_messageNotifier;
        m_messageNotifier = NULL;
        wpa_ctrl_detach(m_monitorConnection);
        wpa_ctrl_close(m_monitorConnection);
        m_monitorConnection = NULL;
    }

    printf("Trying to connect to '%s'\n", cfile);
    m_controlConnection = wpa_ctrl_open(cfile);
    if (m_controlConnection == NULL) {
        delete cfile;
        return -1;
    }
    m_wpaStarted = true;
    //fileStartWpaAction->setDisabled(true);
    fileStopWpaAction->setEnabled(true);
    m_monitorConnection = wpa_ctrl_open(cfile);
    delete cfile;
    if (m_monitorConnection == NULL) {
        wpa_ctrl_close(m_controlConnection);
        return -1;
    }
    if (wpa_ctrl_attach(m_monitorConnection)) {
        printf("Failed to attach to wpa_supplicant\n");
        wpa_ctrl_close(m_monitorConnection);
        m_monitorConnection = NULL;
        wpa_ctrl_close(m_controlConnection);
        m_controlConnection = NULL;
        return -1;
    }

    m_messageNotifier = new QSocketNotifier(wpa_ctrl_get_fd(m_monitorConnection),QSocketNotifier::Read, this);
    if (m_messageNotifier)
        connect(m_messageNotifier, SIGNAL(activated(int)), SLOT(receiveMsgs()));

    return 0;
    
}


int KWlanMainWin::ctrlRequest(const char *command, char *buf, size_t *buflen)
{
    int ret;
    
    if (m_controlConnection == NULL)
        return -1;
    ret = wpa_ctrl_request(m_controlConnection, command, strlen(command), buf, buflen, kwlan_msg_cb);
    
    return ret;
}

void KWlanMainWin::checkConnection()
{
    char buf[10];
    size_t len;
    QString status;
    len = sizeof(buf)-1;
    if (ctrlRequest("PING",buf,&len) <0){
        status =i18n("Connection to wpa_Supplicant lost!");
        txtStatus->setText(status);
        emit statusChanged( false);
        if (openControlConnection(m_controlInterface) >=0){
           status=i18n("Connection to wpa_supplicant established.");
            txtStatus->setText(status);
            m_networkChange = true;
        }
    }
    updateNetworks();
    updateStatus(); 
    //m_tray->setStatus(m_controlInterface, m_associated);
}

void KWlanMainWin::updateNetworks()
{
    char buf[2048], *start, *end, *id, *ssid, *bssid, *flags;
    size_t len;
    int first_active = -1;
    bool selected = false;

    if (!m_networkChange)
        return;
    lbProfiles->clear();
    m_tray->clearProfiles();

    if (m_controlConnection == NULL)
        return;
    
    len = sizeof(buf) - 1;
    if (ctrlRequest("LIST_NETWORKS", buf, &len) < 0)
        return;
    
    buf[len] = '\0';
    start = strchr(buf, '\n');
    if (start == NULL)
        return;
    start++;

    while (*start) {
        bool last = false;
        end = strchr(start, '\n');
        if (end == NULL) {
            last = true;
            end = start;
            while (end[0] && end[1])
                end++;
        }
        *end = '\0';
	
        id = start;
        ssid = strchr(id, '\t');
        if (ssid == NULL)
            break;
        *ssid++ = '\0';
        bssid = strchr(ssid, '\t');
        if (bssid == NULL)
            break;
        *bssid++ = '\0';
        flags = strchr(bssid, '\t');
        if (flags == NULL)
            break;
        *flags++ = '\0';
	
        QString network(id);
        network.append(": ");
        network.append(ssid);
        lbProfiles->insertItem(network);
        m_tray->insertProfile(new QString(network));
	
        if (strstr(flags, "[CURRENT]")) {
            lbProfiles->setCurrentItem(lbProfiles->count() - 1);
            selected = true;
        } else if (first_active < 0 && strstr(flags, "[DISABLED]") == NULL)
            first_active = lbProfiles->count() - 1;
	
            if (last)
                break;
            start = end + 1;
    }

    //if (!selected && first_active >= 0)
    //   lbProfiles->setCurrentItem(first_active);
    m_networkChange = false;
}

void KWlanMainWin::updateStatus()
{
    char buf[2048], *start, *end, *pos;
    size_t len;
    QString wpaError=i18n("Cannot connect to wpa_supplicant.\n"
            "Either it is not started for the selected Interface, or you have no permission to access it.\n"
            "Is ctrl_interface_group defined in wpa_supplicant.conf?");


    len = sizeof(buf) - 1;
    if (m_controlConnection == NULL || ctrlRequest("STATUS", buf, &len) < 0) {
        if (!m_wpaWarned){
            m_wpaWarned=TRUE;
            KMessageBox::sorry(this,wpaError);
        }
        fileStopWpaAction->setDisabled(TRUE);
        fileStartWpaAction->setEnabled(TRUE);
        txtStatus->setText(i18n("Could not get status from wpa_supplicant.\n"));
        txtAuthentication->clear();
        txtEncryption->clear();
        txtSSID->clear();
        txtBSSID->clear();
        txtIPAddress->clear();
        emit statusChanged( false);
        return;
    }
    // Conenction is established, so reset wpaWarned;
    fileStopWpaAction->setEnabled(TRUE);
    fileStartWpaAction->setDisabled(TRUE);
    m_wpaWarned = FALSE;
    buf[len] = '\0';
    
    bool auth_updated = false, ssid_updated = false;
    bool bssid_updated = false, ipaddr_updated = false;
    bool status_updated = false;
    char *pairwise_cipher = NULL, *group_cipher = NULL;
    
    start = buf;
    while (*start) {
        bool last = false;
        end = strchr(start, '\n');
        if (end == NULL) {
            last = true;
            end = start;
            while (end[0] && end[1])
                end++;
        }
        *end = '\0';
	
        pos = strchr(start, '=');
        if (pos) {
            *pos++ = '\0';
            if (strcmp(start, "bssid") == 0) {
                bssid_updated = true;
                txtBSSID->setText(pos);
            } else if (strcmp(start, "ssid") == 0) {
                ssid_updated = true;
                txtSSID->setText(pos);
                m_ssid=txtSSID->text();
            } else if (strcmp(start, "ip_address") == 0) {
                ipaddr_updated = true;
                txtIPAddress->setText(pos);
                if (txtIPAddress->text() =="") m_IpAddress = false;
                else m_IpAddress = true;
                emit ipChanged( txtIPAddress->text() );
            } else if (strcmp(start, "wpa_state") == 0) {
                status_updated = true;
                //QString OldStatus = txtStatus->text();
                txtStatus->setText(pos);
            } else if (strcmp(start, "key_mgmt") == 0) {
                auth_updated = true;
                txtAuthentication->setText(pos);
                /* TODO: could add EAP status to this */
            } else if (strcmp(start, "pairwise_cipher") == 0) {
                pairwise_cipher = pos;
            } else if (strcmp(start, "group_cipher") == 0) {
                group_cipher = pos;
            }
        }
	
        if (last)
            break;
        start = end + 1;
    }
    
    if (pairwise_cipher || group_cipher) {
        QString encr;
        if (pairwise_cipher && group_cipher &&
            strcmp(pairwise_cipher, group_cipher) != 0) {
            encr.append(pairwise_cipher);
            encr.append(" + ");
            encr.append(group_cipher);
            } else if (pairwise_cipher) {
                encr.append(pairwise_cipher);
            } else if (group_cipher) {
                encr.append(group_cipher);
                encr.append(" [group key only]");
            } else {
                encr.append("?");
            }
            txtEncryption->setText(encr);
    } else
        txtEncryption->clear();

        if (!status_updated)
            txtStatus->clear();
        if (!auth_updated)
            txtAuthentication->clear();
        if (!ssid_updated)
            txtSSID->clear();
        if (!bssid_updated)
            txtBSSID->clear();
        if (!ipaddr_updated)
        {
            txtIPAddress->clear();
            emit(ipChanged(""));
        }
}

void KWlanMainWin::profileDelete()
{
    char cmd[256],reply[100];
    size_t reply_len;
    // get Profile
    QListBoxItem *selected = lbProfiles->selectedItem();
    // check if a profile is selected
    if (!selected) {
        KMessageBox::sorry(this,i18n("No Profile selected!"));
        return;
    }
    
    QString sel(lbProfiles->currentText());
    int pos = sel.find(':');
    if (pos < 0) {
        KMessageBox::sorry(this,i18n("Invalid Profile!"));
        return;
    }
    sel.truncate(pos);
    int answer = KMessageBox::warningYesNo(this,i18n("Do you really want to delete this profile?"));
    if (answer==KMessageBox::Yes){
        snprintf(cmd, sizeof(cmd), "REMOVE_NETWORK %s", sel.ascii());
        reply_len = sizeof(reply);
        ctrlRequest(cmd, reply, &reply_len);
        if (strncmp(reply, "OK", 2) != 0) {
            KMessageBox::sorry(this, i18n("Failed to remove network from wpa_supplicant\nconfiguration."));
        } else {
            triggerUpdate();
            reply_len = sizeof(reply);
            ctrlRequest("SAVE_CONFIG", reply, &reply_len);
            if (strncmp(reply,"OK",2) != 0) {
                KMessageBox::sorry(this, "Failed to save the  wpa_supplicant configuration.\nIs update_config=1 defined in wpa_supplicant.conf?");
            }        }
    }
}

void KWlanMainWin::profileModify()
{
    // get Profile
    QListBoxItem *selected = lbProfiles->selectedItem();
    // check if a profile is selected
    if (!selected) {
        KMessageBox::sorry(this,i18n("No Profile selected!"));
        return;
    }
    
    QString sel(lbProfiles->currentText());
    int pos = sel.find(':');
    if (pos < 0) {
        KMessageBox::sorry(this,i18n("Invalid Profile!"));
        return;
    }
    sel.truncate(pos);
    
    
    // Open Config Dialog
    KProfileConfigDlg *profileDlg = new KProfileConfigDlg();
    if (!profileDlg) return;
    profileDlg->setMainWin( this);
    profileDlg->paramsFromConfig(sel.toInt());
    profileDlg->show();
    profileDlg->exec();
    
}

void KWlanMainWin::profileNew()
{
    // Open Config Dialog
    KProfileConfigDlg *profileDlg = new KProfileConfigDlg();
    if (!profileDlg) return;
    profileDlg->setMainWin( this);
    profileDlg->newNetwork();
    profileDlg->show();
    profileDlg->exec();
}

void KWlanMainWin::profileActivate()
{
    // get Profile
    QListBoxItem *selected = lbProfiles->selectedItem();
    // check if a profile is selected
    if (!selected) {
        KMessageBox::sorry(this,i18n("No Profile selected!"));
        return;
    }
    // release current ip address if associated
    if (m_dhcpEnabled && m_associated) releaseIpAddress();
    
    QString sel(lbProfiles->currentText());
    profileSelect(sel);
    checkConnection();
}

void KWlanMainWin::profileSelect(QString profile)
{
    int ret;
    char reply[40], cmd[256];
    //size_t len;
    size_t reply_len;
    QString ssid, ipAddress,netmask,dns1,dns2,gateway, domain;
    QString sel = profile;
    bool dhcpEnabled;
    bool dontOverrideDns,dontOverrideGw;
    int pos = sel.find(':');
    if (pos < 0) {
        KMessageBox::sorry(this,i18n("Invalid Profile!"));
        return;
    }
    sel.truncate(pos);
    // Disconnect from current network
    snprintf(cmd, sizeof(cmd), "DISCONNECT");
    reply_len = sizeof(reply);
    ctrlRequest(cmd, reply, &reply_len);
 
    // Enable Network
    snprintf(cmd, sizeof(cmd), "SELECT_NETWORK %d", sel.toInt());
    reply_len = sizeof(reply);
    ret=ctrlRequest(cmd, reply, &reply_len);
    if (ret==-1) KMessageBox::sorry(this,"No control connection!");
    if (strncmp(reply, "OK", 2) != 0) {
        KMessageBox::sorry(this, i18n("Failed to enable network in wpa_supplicant\nconfiguration."));
    } 
    // store in config
    Config().m_lastSSID=profile;
    // set ip data
    sel = profile;
    ssid = sel.right(sel.length()-pos-2);
    ret = Config().readIpSettings(ssid,dhcpEnabled, ipAddress, netmask, gateway, dns1, dns2,domain,dontOverrideGw,dontOverrideDns);
    m_ssid = ssid;
    if ((ret != 0)|| (dhcpEnabled)){
        m_dhcpEnabled = true; //default will be dhcp
        m_staticIpAddress="";
        m_staticNetmask = "";
        m_staticGateway="";
        m_staticDns1 = "";
        m_staticDns2="";
        m_dontOverrideDns="";
        m_dontOverrideGw="";
        m_domain = "";
    }
    else {
        // set static addresses;
        m_dhcpEnabled = dhcpEnabled;
        m_staticIpAddress= ipAddress;
        m_staticNetmask = netmask;
        m_staticGateway=gateway;
        m_staticDns1 = dns1;
        m_staticDns2=dns2;
        m_dontOverrideGw=dontOverrideGw;
        m_dontOverrideDns=dontOverrideDns;
        m_domain=domain;
    }
    // Connect
    snprintf(cmd, sizeof(cmd), "REASSOCIATE");
    reply_len = sizeof(reply);
    ctrlRequest(cmd, reply, &reply_len);
}

void KWlanMainWin::helpAbout()
{
    KAboutApplication about(this);
    about.exec();
}


void KWlanMainWin::fileExit()
{
    //delete m_tray;
    Config().writeSettings();
    kdDebug() << "Exiting..." << endl;
    kapp->quit();
}




#include "kwlanmainwin.moc"




















